#include <amxmodx>
#include <ze_core>
#include <ze_levels>

// Enum
enum _:RANKS
{
	RANK_START = 0,
	RANK_END,
	RANK_NAME[MAX_NAME_LENGTH]
}

// Cvars.
new g_iMaxLevels

// Variable.
new g_iForward

// String.
new g_szRankName[MAX_PLAYERS+1][MAX_NAME_LENGTH]

// Dynamic Array.
new Array:g_aRanks

public plugin_natives()
{
	register_native("ze_get_user_rank", "__native_get_user_rank")
	register_native("ze_set_user_rank", "__native_set_user_rank")
}

public plugin_init()
{
	// Load Plug-In.
	register_plugin("[ZE] Addon: Level Ranks", "1.0", "z0h1r-LK")

	// Cvars.
	bind_pcvar_num(register_cvar("ze_levels_maximum", "100"), g_iMaxLevels)

	// Create Forward.
	g_iForward = CreateMultiForward("ze_fw_rank_changed", ET_IGNORE, FP_CELL, FP_STRING, FP_CELL)

	// Create new Array.
	g_aRanks = ArrayCreate(RANKS, 1)
}

public plugin_end()
{
	// Free the Memory.
	ArrayDestroy(g_aRanks)
	DestroyForward(g_iForward)
}

public plugin_cfg()
{
	new szCfgDir[24], szFile[64]
	get_configsdir(szCfgDir, charsmax(szCfgDir))
	formatex(szFile, charsmax(szFile), "%s/%s.ini", szCfgDir, ZE_FILENAME)
	read_Data(szFile)
}

public client_putinserver(id)
{
	if (is_user_hltv(id))
		return

	set_task(0.5, "read_ClientRank", id)
}

public read_ClientRank(id)
{
	// Player disconnected?
	if (!is_user_connected(id))
		return

	get_Rank(id, ze_get_user_level(id), 0)
}

public client_disconnected(id, bool:drop, message[], maxlen)
{
	if (is_user_hltv(id))
		return

	g_szRankName[id] = NULL_STRING
}

public ze_fw_user_level(id, iLevel)
{
	get_Rank(id, iLevel, 1)
}

public get_Rank(id, iLevel, updated)
{
	new aArray[RANKS], iSizeNum = ArraySize(g_aRanks)

	if (iLevel >= g_iMaxLevels)
	{
		ArrayGetArray(g_aRanks, iSizeNum - 1, aArray)
		copy(g_szRankName[id], charsmax(g_szRankName[]), aArray[RANK_NAME])
	}
	else
	{
		for (new i = 0; i < iSizeNum; i++)
		{
			ArrayGetArray(g_aRanks, i, aArray)

			if (aArray[RANK_START] <= (iLevel) <= aArray[RANK_END])
			{
				copy(g_szRankName[id], charsmax(g_szRankName[]), aArray[RANK_NAME])
				break
			}
		}
	}

	if (updated)
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "MSG_NEW_RANKNAME", g_szRankName[id])

		// Call forward ze_fw_rank_changed(param, string[]).
		ExecuteForward(g_iForward, _/* Ignore return value */, id, g_szRankName[id], iLevel)
	}
}

/**
 * -=| Function |=-
 */
read_Data(const szFile[])
{
	new hFile  // Open the file.
	if ((hFile = fopen(szFile, "a+t")))
	{
		new szRead[256], szSect[64], bFound, iLine
		while (!feof(hFile))
		{
			iLine++
			szRead = NULL_STRING

			if (!fgets(hFile, szRead, charsmax(szRead)))
				continue

			trim(szRead), replace(szRead, charsmax(szRead), "^n", "")

			switch (szRead[0])
			{
				case '#', ';', 0:  // Comment or empty line.
				{
					continue
				}
				case '[':  // New Section.
				{
					szSect = NULL_STRING

					if (copyc(szSect, charsmax(szSect), szRead[1], ']'))
					{
						if (equal(szSect, "Level Ranks"))
						{
							bFound = 1
							break
						}
					}
				}
			}
		}

		if (!bFound)
		{
			fprintf(hFile, "[%s]^n# Format usage: <start-end> <rank name>", szSect)
			fclose(hFile)
			return true
		}

		new aArray[RANKS], szRankName[64], szLevels[34], szStart[16], szEnd[16]
		while (!feof(hFile))
		{
			iLine++
			szRead = NULL_STRING

			if (!fgets(hFile, szRead, charsmax(szRead)))
				break

			trim(szRead), replace(szRead, charsmax(szRead), "^n", "")

			switch (szRead[0])
			{
				case '#', ';', 0:
				{
					continue
				}
				case ']': // New Section.
				{
					break
				}
			}

			szLevels = NULL_STRING
			szRankName = NULL_STRING

			if (parse(szRead, szLevels, charsmax(szLevels), szRankName, charsmax(szRankName)) < 2)
			{
				server_print("[ZE][Ranks] Error format • Line #%i", iLine)
				continue
			}

			remove_quotes(szLevels)
			remove_quotes(szRankName)

			if (szLevels[0] && szRankName[0])
			{
				strtok2(szLevels, szStart, charsmax(szStart), szEnd, charsmax(szEnd), '-', 1)

				aArray[RANK_START] = str_to_num(szStart)
				aArray[RANK_END]   = str_to_num(szEnd)
				copy(aArray[RANK_NAME], charsmax(aArray) - RANK_NAME, szRankName)

				ArrayPushArray(g_aRanks, aArray)
			}
			else
			{
				server_print("[ZE][Ranks] Error in argument • Line #%i", iLine)
			}
		}

		fclose(hFile)
		return true
	}

	return false
}

/**
 * -=| Natives |=-
 */
public __native_get_user_rank(plugin_id, num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return 0
	}

	return set_string(2, g_szRankName[id], get_param(3))
}

public __native_set_user_rank(plugin_id, num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return false
	}

	new szRankName[MAX_NAME_LENGTH]
	if (!get_string(1, szRankName, charsmax(szRankName)))
		g_szRankName[id] = NULL_STRING
	else
		copy(g_szRankName[id], charsmax(g_szRankName[]), szRankName)
	return true
}